package ru.landgrafhomyak.bgtu.networks0.build_script

import java.util.function.BiFunction
import kotlin.properties.ReadOnlyProperty
import kotlin.reflect.KProperty
import org.gradle.api.Action
import org.gradle.api.NamedDomainObjectContainer
import org.gradle.api.NamedDomainObjectProvider
import org.gradle.api.Transformer
import org.gradle.api.provider.Provider
import org.gradle.api.specs.Spec
import org.jetbrains.kotlin.gradle.plugin.KotlinSourceSet

@Suppress("FunctionName")
private fun NamedDomainObjectContainer<KotlinSourceSet>._createConditional(
	name: String,
	vararg children: KotlinSourceSet?
): KotlinSourceSet? {
	val existingChildren = children.filterNotNull()

	if (existingChildren.isEmpty())
		return null

	val ss = this.create(name)
	existingChildren
		.forEach { child -> child.dependsOn(ss) }

	return ss
}

fun NamedDomainObjectContainer<KotlinSourceSet>.setupHierarchy() {
	val linuxX64Main = this.findByName("linuxX64Main")
	val linuxX64Test = this.findByName("linuxX64Test")
	val linuxArm64Main = this.findByName("linuxArm64Main")
	val linuxArm64Test = this.findByName("linuxArm64Test")

	val linuxMain = this._createConditional("linuxMain", linuxX64Main, linuxArm64Main)
	val linuxTest = this._createConditional("linuxTest", linuxX64Test, linuxArm64Test)

	val macosX64Main = this.findByName("macosX64Main")
	val macosX64Test = this.findByName("macosX64Test")
	val macosArm64Main = this.findByName("macosArm64Main")
	val macosArm64Test = this.findByName("macosArm64Test")

	val macosMain = this._createConditional("macosMain", macosX64Main, macosArm64Main)
	val macosTest = this._createConditional("macosTest", macosX64Test, macosArm64Test)

	val posixMain = this._createConditional("posixMain", linuxMain, macosMain)
	val posixTest = this._createConditional("posixTest", linuxTest, macosTest)

	val mingwX64Main = this.findByName("mingwX64Main")
	val mingwX64Test = this.findByName("mingwX64Test")

	val windowsMain = this._createConditional("windowsMain", mingwX64Main)
	val windowsTest = this._createConditional("windowsTest", mingwX64Test)

	val nativeMain = this._createConditional("nativeMain", posixMain, windowsMain)
	val nativeTest = this._createConditional("nativeTest", posixTest, windowsTest)

	nativeMain?.dependsOn(this.getByName("commonMain"))
	nativeTest?.dependsOn(this.getByName("commonTest"))
}


val NamedDomainObjectContainer<KotlinSourceSet>.posixMain by SourceSetDelegate
val NamedDomainObjectContainer<KotlinSourceSet>.posixTest by SourceSetDelegate
val NamedDomainObjectContainer<KotlinSourceSet>.windowsMain by SourceSetDelegate
val NamedDomainObjectContainer<KotlinSourceSet>.windowsTest by SourceSetDelegate


private object SourceSetDelegate : ReadOnlyProperty<NamedDomainObjectContainer<KotlinSourceSet>, NamedDomainObjectProvider<KotlinSourceSet>> {
	private class BoundProvider(
		private val _collection: NamedDomainObjectContainer<KotlinSourceSet>,
		private val _name: String
	) : NamedDomainObjectProvider<KotlinSourceSet> {
		override fun get(): KotlinSourceSet = this._collection.getByName(this._name)

		override fun getOrNull(): KotlinSourceSet? = this._collection.findByName(this._name)

		override fun isPresent(): Boolean = this.getOrNull() != null

		@Deprecated("Deprecated in Java")
		override fun forUseAtConfigurationTime(): Provider<KotlinSourceSet> {
			TODO("Not yet implemented")
		}

		override fun getName(): String = this._name

		override fun configure(action: Action<in KotlinSourceSet>) {
			action.execute(this.get())
		}

		override fun <U : Any?, R : Any?> zip(right: Provider<U>, combiner: BiFunction<in KotlinSourceSet, in U, out R?>): Provider<R> {
			TODO("Not yet implemented")
		}

		override fun orElse(provider: Provider<out KotlinSourceSet>): Provider<KotlinSourceSet> {
			TODO("Not yet implemented")
		}

		override fun orElse(value: KotlinSourceSet): Provider<KotlinSourceSet> {
			TODO("Not yet implemented")
		}

		override fun <S : Any?> flatMap(transformer: Transformer<out Provider<out S>?, in KotlinSourceSet>): Provider<S> {
			TODO("Not yet implemented")
		}

		override fun filter(spec: Spec<in KotlinSourceSet>): Provider<KotlinSourceSet> {
			TODO("Not yet implemented")
		}

		override fun <S : Any?> map(transformer: Transformer<out S?, in KotlinSourceSet>): Provider<S> {
			TODO("Not yet implemented")
		}

		override fun getOrElse(defaultValue: KotlinSourceSet): KotlinSourceSet = this.getOrNull() ?: defaultValue
	}

	override fun getValue(thisRef: NamedDomainObjectContainer<KotlinSourceSet>, property: KProperty<*>): NamedDomainObjectProvider<KotlinSourceSet> {
		return BoundProvider(thisRef, property.name)
	}
}