package ru.landgrafhomyak.bgtu.networks0.icmp

import ru.landgrafhomyak.utility.IntSerializers

@OptIn(ExperimentalUnsignedTypes::class)
class IcmpPingHeader(
	val identifier_be: UShort,
	val seqNo_be: UShort,
) : IcmpHeader {
	override val type: UByte get() = 8u
	override val code: UByte get() = 0u

	override fun checksum(withData: UByteArray): UShort = Ipv4Checksum.calculate { c ->
		c.addFirstByte(this.type)
		c.addSecondByte(this.code)
		c.addWord(this.identifier_be)
		c.addWord(this.seqNo_be)
		c.addTrailingData(withData)
	}

	override fun serialize(withData: UByteArray): UByteArray {
		val serialized = UByteArray(withData.size + 8)
		IntSerializers.encode8Bu(serialized, 0, this.type)
		IntSerializers.encode8Bu(serialized, 1, this.code)
		IntSerializers.encode16beHu(serialized, 2, this.checksum(withData))
		IntSerializers.encode16beHu(serialized, 4, this.identifier_be)
		IntSerializers.encode16beHu(serialized, 6, this.seqNo_be)
		withData.copyInto(serialized, 8)
		return serialized
	}
}