package ru.landgrafhomyak.bgtu.networks0.icmp

import kotlin.contracts.ExperimentalContracts
import kotlin.contracts.InvocationKind
import kotlin.contracts.contract

@OptIn(ExperimentalUnsignedTypes::class)
internal class Ipv4Checksum() {
	private var _accumulator: UInt = 0u

	fun addFirstByte(b: UByte) {
		this._accumulator += (b.toUInt() shl 8)
	}

	fun addSecondByte(b: UByte) {
		this._accumulator += b
	}

	fun addWord(b: UShort) {
		this._accumulator += b
	}

	fun addLastByte(b: UByte) {
		this._accumulator += b
	}

	fun addTrailingData(data: UByteArray) {
		for (i in data.indices step 2) {
			this.addFirstByte(data[i])
			this.addSecondByte(data[i + 1])
		}
		if (data.size % 2 != 0)
			this.addLastByte(data.last())
	}

	fun export(): UShort {
		var out: UInt = this._accumulator
		while (out > 0xFFFFu) {
			out = (out and 0xFFFFu) + (out shr 16);
		}
		return out.toUShort().inv()
	}

	companion object {
		@OptIn(ExperimentalContracts::class)
		inline fun calculate(builder: (Ipv4Checksum) -> Unit): UShort {
			contract {
				callsInPlace(builder, InvocationKind.EXACTLY_ONCE)
			}

			val b = Ipv4Checksum()
			builder(b)
			return b.export()
		}
	}
}