package ru.landgrafhomyak.bgtu.networks0.low_level.c_interop_utilities


import kotlinx.cinterop.CPointer
import kotlinx.cinterop.CVariable
import kotlinx.cinterop.ExperimentalForeignApi
import platform.posix.malloc
import kotlinx.cinterop.sizeOf

@OptIn(ExperimentalForeignApi::class)
object CUtilities {
	/**
	 * Returns size of a C type in bytes, but as [unsigned long long][ULong].
	 * @see sizeOf
	 * @see sizeOfUI
	 */
	inline fun <reified T : CVariable> sizeOfUL(): ULong = sizeOf<T>().toULong()

	/**
	 * Returns size of a C type in bytes, but as [unsigned long][UInt].
	 * @see sizeOf
	 * @see sizeOfUL
	 */
	inline fun <reified T : CVariable> sizeOfUI(): UInt = sizeOf<T>().toUInt()

	/**
	 * Returns size of a C type in bytes, but as [unsigned long][UInt].
	 * @see sizeOf
	 * @see sizeOfUL
	 */
	@Suppress("UNCHECKED_CAST")
	inline fun <reified T : CVariable> newOrOom() = (malloc(sizeOfUL<T>()) ?: throw OutOfMemoryError()) as CPointer<T>
}