package ru.landgrafhomyak.bgtu.networks0.low_level.c_interop_utilities

import kotlin.contracts.ExperimentalContracts
import kotlin.contracts.InvocationKind
import kotlin.contracts.contract
import kotlinx.cinterop.ExperimentalForeignApi
import kotlinx.cinterop.toKStringFromUtf8
import platform.posix.errno
import platform.posix.strerror


object PosixUtilities {
	@OptIn(ExperimentalForeignApi::class, ExperimentalContracts::class)
	inline fun throwErrno(
		createError: (errnoDescription: String) -> Throwable,
	): Nothing {
		contract {
			callsInPlace(createError, InvocationKind.AT_MOST_ONCE)
		}
		val origErrno = errno
		throwErrno(origErrno, createError)
	}

	@OptIn(ExperimentalForeignApi::class, ExperimentalContracts::class)
	inline fun throwErrno(
		origErrno: Int,
		createError: (errnoDescription: String) -> Throwable,
	): Nothing {
		contract {
			callsInPlace(createError, InvocationKind.AT_MOST_ONCE)
		}
		val raw = strerror(origErrno)
		if (raw != null) {
			throw createError(raw.toKStringFromUtf8()) // todo handle error
		} else {
			val strerrErrno = errno
			try {
				throw createError("Unknown error (errno=${origErrno})")
			} catch (t: Throwable) {
				t.addSuppressed(RuntimeException("Failed to get errno description (failed with errno=${strerrErrno})"))
				throw t
			}
		}
	}
}