package ru.landgrafhomyak.bgtu.networks0.low_level.multithreading

import ru.landgrafhomyak.bgtu.networks0.utilities.CloseableRefCounter
import ru.landgrafhomyak.bgtu.networks0.utilities.safeAutoClose1
import ru.landgrafhomyak.bgtu.networks0.utilities.safeAutoClose2

class CountDownLatch : AutoCloseable {
	private val _refcnt: CloseableRefCounter
	private var _counter: ULong
	private val _mutex: Mutex
	private val _condition: Condition

	constructor(initialCounterValue: ULong) {
		this._refcnt = CloseableRefCounter("Latch was destroyed")
		this._counter = initialCounterValue
		this._mutex = Mutex()
		safeAutoClose2(onAbort = this._mutex::close) {
			this._condition = Condition()
		}
	}

	fun await() = this._refcnt.withRef {
		this._mutex.withLock {
			if (this._counter == 0uL)
				return
			this._condition.await(this._mutex)
		}
	}

	fun decrement() = this._refcnt.withRef {
		this._mutex.withLock {
			if (this._counter == 0uL) return
			if (--this._counter == 0uL)
				this._condition.signalAll()
		}
	}

	override fun close() {
		this._refcnt.close("Latch is still in use")
		safeAutoClose1(
			action = this._condition::close,
			finally = this._mutex::close
		)
	}
}