package ru.landgrafhomyak.bgtu.networks0.low_level.multithreading

import kotlinx.cinterop.CPointer
import kotlinx.cinterop.ExperimentalForeignApi
import kotlinx.cinterop.alloc
import kotlinx.cinterop.free
import kotlinx.cinterop.nativeHeap
import kotlinx.cinterop.ptr
import platform.posix.pthread_mutex_destroy
import platform.posix.pthread_mutex_init
import platform.posix.pthread_mutex_lock
import platform.posix.pthread_mutex_t
import platform.posix.pthread_mutex_unlock
import ru.landgrafhomyak.bgtu.networks0.low_level.c_interop_utilities.PosixUtilities
import ru.landgrafhomyak.bgtu.networks0.utilities.CloseableRefCounter
import ru.landgrafhomyak.bgtu.networks0.utilities.safeAutoClose1
import ru.landgrafhomyak.bgtu.networks0.utilities.safeAutoClose2

@OptIn(ExperimentalForeignApi::class, Mutex.RefcntAccess::class)
actual class Mutex : AutoCloseable {
	@RequiresOptIn
	internal annotation class RefcntAccess

	@RefcntAccess
	internal val _refcnt: CloseableRefCounter
	internal val _descriptor: CPointer<pthread_mutex_t>

	actual constructor() {
		this._refcnt = CloseableRefCounter("Pthreads mutex was destroyed")
		this._descriptor = nativeHeap.alloc<pthread_mutex_t>().ptr
		safeAutoClose2(onAbort = { nativeHeap.free(this._descriptor) }) {
			var err = pthread_mutex_init(this._descriptor, null)
			if (err != 0)
				PosixUtilities.throwErrno(err) { d -> RuntimeException("Failed to create pthreads mutex: $d") }
		}
	}

	private fun _throwClosed(): Nothing =
		throw IllegalStateException()


	actual fun lock() = this._refcnt.tryIncref {
		var err = pthread_mutex_lock(this._descriptor)
		if (err != 0)
			PosixUtilities.throwErrno(err) { d -> RuntimeException("Failed to lock pthreads mutex: $d") }
	}

	actual fun unlock() = this._refcnt.tryDecref {
		var err = pthread_mutex_unlock(this._descriptor)
		if (err != 0)
			PosixUtilities.throwErrno(err) { d -> RuntimeException("Failed to unlock pthreads mutex: $d") }
	}

	override fun close() {
		this._refcnt.close("There are waiting threads on this pthreads mutex")
		safeAutoClose1(
			action = {
				var err = pthread_mutex_destroy(this._descriptor)
				if (err != 0)
					PosixUtilities.throwErrno(err) { d -> RuntimeException("Failed to destroy pthreads mutex: $d") }
			},
			finally = { nativeHeap.free(this._descriptor) }
		)
	}
}