package ru.landgrafhomyak.bgtu.networks0.low_level.sockets

/**
 * High-level interface for interacting with icmp socket.
 * Destination host address is bound to instance and can't be changed.
 */
interface IcmpSocket : AutoCloseable {
	/**
	 * Sends [data] as is in single datagram to destination host.
	 */
	suspend fun sendRaw(data: UByteArray)

	/**
	 * Waits for datagram from destination host and returns it as is.
	 */
	suspend fun recvRaw(): UByteArray

	/**
	 * Closes socket and releases related resources.
	 * @throws IllegalStateException if called on already closed socket.
	 */
	override fun close()
}