package ru.landgrafhomyak.bgtu.networks0.low_level.sockets

/**
 * User-side interface of socket event-loops.
 */
@Suppress("FunctionName")
interface SocketEventLoopScope {
	/**
	 * Creates [ICMP socket][IcmpSocket] bound with specified [address][addr] to communicate over IPv4.
	 */
	fun icmp_IPv4(addr: String): IcmpSocket

	/**
	 * Creates [ICMP socket][IcmpSocket] bound with specified [address][addr] to communicate over IPv6.
	 */
	fun icmp_IPv6(addr: String): IcmpSocket

	interface Closeable : SocketEventLoopScope, AutoCloseable {
		/**
		 * Closes event-loop and releases related resources.
		 *
		 * @throws IllegalStateException if there are unclosed sockets bound to this event-loop.
		 * @throws IllegalStateException if called on already closed socket.
		 */
		override fun close()
	}
}