@file:OptIn(ExperimentalContracts::class)

package ru.landgrafhomyak.bgtu.networks0.utilities

import kotlin.contracts.ExperimentalContracts
import kotlin.contracts.InvocationKind
import kotlin.contracts.contract

@Suppress("WRONG_INVOCATION_KIND")
inline fun <R> safeAutoClose1(
	finally: () -> Unit,
	action: () -> R
): R {
	contract {
		callsInPlace(action, InvocationKind.EXACTLY_ONCE)
		callsInPlace(finally, InvocationKind.EXACTLY_ONCE)
	}
	return safeAutoClose3(onAbort = finally, onSuccess = finally, onCrossReturn = finally, action = action)
}

@Suppress("WRONG_INVOCATION_KIND")
inline fun <R> safeAutoClose2(
	onAbort: () -> Unit = {},
	onSuccess: () -> Unit = {},
	action: () -> R
): R {
	contract {
		callsInPlace(action, InvocationKind.EXACTLY_ONCE)
		callsInPlace(onAbort, InvocationKind.AT_MOST_ONCE)
		callsInPlace(onSuccess, InvocationKind.AT_MOST_ONCE)
	}
	return safeAutoClose3(onAbort = onAbort, onSuccess = onSuccess, onCrossReturn = onSuccess, action = action)
}

@Suppress("WRONG_INVOCATION_KIND")
inline fun <R> safeAutoClose2e(
	onAbort: (Throwable) -> Unit = { _ -> },
	onSuccess: () -> Unit = {},
	action: () -> R
): R {
	contract {
		callsInPlace(action, InvocationKind.EXACTLY_ONCE)
		callsInPlace(onAbort, InvocationKind.AT_MOST_ONCE)
		callsInPlace(onSuccess, InvocationKind.AT_MOST_ONCE)
	}
	return safeAutoClose3e(onAbort = onAbort, onSuccess = onSuccess, onCrossReturn = onSuccess, action = action)
}

inline fun <R> safeAutoClose3(
	onAbort: () -> Unit = {},
	onSuccess: () -> Unit = {},
	onCrossReturn: () -> Unit = {},
	action: () -> R
): R {
	contract {
		callsInPlace(action, InvocationKind.EXACTLY_ONCE)
		callsInPlace(onAbort, InvocationKind.AT_MOST_ONCE)
		callsInPlace(onSuccess, InvocationKind.AT_MOST_ONCE)
		callsInPlace(onCrossReturn, InvocationKind.AT_MOST_ONCE)
	}
	return safeAutoClose3e(onAbort = { t -> onAbort() }, onSuccess = onSuccess, onCrossReturn = onCrossReturn, action = action)
}

inline fun <R> safeAutoClose3e(
	onAbort: (Throwable) -> Unit = { _ -> },
	onSuccess: () -> Unit = {},
	onCrossReturn: () -> Unit = {},
	action: () -> R
): R {
	contract {
		callsInPlace(action, InvocationKind.EXACTLY_ONCE)
		callsInPlace(onAbort, InvocationKind.AT_MOST_ONCE)
		callsInPlace(onSuccess, InvocationKind.AT_MOST_ONCE)
		callsInPlace(onCrossReturn, InvocationKind.AT_MOST_ONCE)
	}

	val ret: R
	var wasError = false
	var crossReturned = true
	try {
		ret = action()
		crossReturned = false
	} catch (e1: Throwable) {
		wasError = true
		try {
			onAbort(e1)
		} catch (e2: Throwable) {
			e1.addSuppressed(e2)
		}
		throw e1
	} finally {
		if (!wasError) {
			if (crossReturned)
				onCrossReturn()
			else
				onSuccess()
		}
	}
	return ret
}